﻿----------------------------------------
-- Epic Gem List v 0.2, Copyright 2010 Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	Added blue cataclysm gems with test on first load
-- 0.3  Added JC only gems, cataclysm green and WotLK blue gems
-- 0.4	fixed loading of missing gems on first start (delayed GetItemInfo)
-- 0.5	added french translation, thanks to Araldwenn
-- 1.0	most translations from GlobalStrings.lua, quality and level selection, source cleanup, minor fixes
-- 1.1	new design, more detailed selections
----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------

EGL_VERSION = "ver 1.1 by Ydy @ Shattrath"

EGL_SOCKETSELECT = _G["COLOR"];
EGL_STATSELECT = _G["STATS_LABEL"];
EGL_CLOSE = _G["CLOSE"];
EGL_JC = EGL_GemLocaleText["jc"];

EGL_SCAN_FRAME_TITLE = "Epic Gem List";
EGL_SCAN_FRAME_SUBTITLE = "Retrieving Gem Data from server ...";
EGL_SCAN_FRAME_ABORT = "cancel";
EGL_SCAN_FRAME_INITIALIZING = "Initializing scan ...";
EGL_QUALITY = _G["QUALITY"];
EGL_ITEMLEVEL = _G["LEVEL"];

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local waitTable = {};
local waitFrame = nil;

-- INITIALIZE UNSET VARS ------------------------------------------------

if(EGL_Options == nil) then
    EGL_Options = {};
end

if ( EGL_Options["SelectedStat"]  == nil ) then
    EGL_Options["SelectedStat"] = 0;
end

if ( EGL_Options["SelectedSlot"] == nil ) then
    EGL_Options["SelectedSlot"] = 0;
end

if ( EGL_Options["SelectedQualiOp"] == nil ) then
    EGL_Options["SelectedQualiOp"] = 1;
end

if ( EGL_Options["SelectedQuali"] == nil ) then
    EGL_Options["SelectedQuali"] = 2;
end

if ( EGL_Options["SelectedLevelOp"] == nil ) then
    EGL_Options["SelectedLevelOp"] = 1;
end

if ( EGL_Options["SelectedLevel"] == nil ) then
    EGL_Options["SelectedLevel"] = 2;
end

if (EGL_Options["jc"] == nil ) then
	EGL_Options["jc"] = false;
	EGL_Options["jc_cata"] = false;
end

if (EGL_Options["jc"] and EGL_Options["cataclysm"]) then
	EGL_Options["jc_cata"] = true;
end
if (EGL_Options["jc"] == false) then
	EGL_Options["jc_cata"] = false
end

if (EGL_ScannedGems == nil) then
	EGL_ScannedGems = 0;
end
if (egl_scan_state == nil) then
	egl_scan_state = 0;
end
if (EGL_GemCount == nil) then
	EGL_GemCount = 0;
end
if (egl_shown_gems == nil) then
	egl_shown_gems = 0;
end

-- MISC FUNCTIONS ------------------------------------------------

---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
	--UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end

---- Delayed script execution ----
function EGL_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Gem List", {
    type = "launcher",
	label = "EpicGemList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EGL_Form:IsVisible() then
		        EGL_Form:Hide()
		    else
		        EGL_Form:Show()
	        end
        end
    end,
	icon = "Interface\\AddOns\\EpicGemList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Gem List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EGL_SlashCommandHandler(msg)
	if (msg == "scan") then
		EGL_ForceScan();
	end
	if EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
end

-- ON LOAD ------------------------------------------------

function EGL_OnLoad(this)
	SELECTEDITEM = 0;
  
	out("EpicGemList Loaded...");
	eglf = _G["EGL_Form"];
	SLASH_EGL1 = "/epicgemlist";
	SLASH_EGL2 = "/egl";
	SlashCmdList["EGL"] = function(msg)
		EGL_SlashCommandHandler(msg);
	end
	if EGL_Form:IsVisible() then
		EGL_Form:Hide()
	else
		EGL_Form:Show()
	end
	EGL_Options = {};
	EGL_Options["SelectedSlot"] = 0;
	EGL_Options["SelectedStat"] = 0;
	EGL_Options["SelectedQualiOp"] = 1;
	EGL_Options["SelectedQuali"] = 2;
	EGL_Options["SelectedLevelOp"] = 1;
	EGL_Options["SelectedLevel"] = 2;
	local version, build, date, tocversion = GetBuildInfo()
	if ((version == "4.0.1") or (version == "4.0.2")) then
		EGL_Options["cataclysm"] = false;
	else
		EGL_Options["cataclysm"] = true;
	end
	EGL_SlotDropDown_Init();
	EGL_StatDropDown_Init();
end

-- ON SHOW ------------------------------------------------

function EGL_OnShow()
	EGL_GemCount = 0;
	for value,_ in pairs(EGL_Data) do
		EGL_GemCount = EGL_GemCount + 1;
	end
	for value,_ in pairs(EGL_Data_JC) do
		EGL_GemCount = EGL_GemCount + 1;
	end
	if (EGL_Options["cataclysm"]) then
		for value,_ in pairs(EGL_Data_Cata) do
			EGL_GemCount = EGL_GemCount + 1;
		end
		for value,_ in pairs(EGL_Data_Cata_JC) do
			EGL_GemCount = EGL_GemCount + 1;
		end
	end
	if egl_scan_state == 0 then
		EGL_GemScan();
	end
end

-- ON EVENT ------------------------------------------------

-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EGL_Help_OnEnter(this)
	local entries = table.getn(EGL_Help);
	EGLTooltip:ClearLines();
	EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EGLTooltip:AddLine(EGL_Help[i]);
	end
    EGLTooltip:Show();
end

function EGL_Help_OnLeave()
  EGLTooltip:Hide();
end

-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EGLItem_OnEnter(this)
  EGLTooltip:ClearLines();
  local id = this:GetID()+ FauxScrollFrame_GetOffset(EGL_ScrollBar);  
  
  if id ~= nil then
  	local iteminfo = GetItemInfo(EGL[id]);
    local itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemCount, itemEquipLoc, itemTexture = GetItemInfo(EGL[id]);

    EGLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
    EGLTooltip:SetHyperlink("item:"..EGL[id]..":0:0:0");
	EGLTooltip:AddLine("Item Level: "..itemLevel);
    EGLTooltip:Show();

	if EnhTooltip and EnhTooltip.TooltipCall then   
	  quantity = 1;
	  EnhTooltip.TooltipCall(EGLTooltip, itemName, itemLink, itemQuality, quantity);
	end
  end 
end

function EGLItem_OnLeave()
  SELECTEDITEM = 0;
  EGLTooltip:Hide();
end

function EGLItem_OnClick(this)
	local EGL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EGL_ScrollBar);
	local iteminfo = GetItemInfo(EGL[id]);
    local itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemCount, itemEquipLoc, itemTexture = GetItemInfo(EGL[id])
	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and iteminfo) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
	-- if (IsAltKeyDown()) then
		-- if (EGL_Data[EGL[id]] ~= nil) then
			-- EGL_Skill_ID = EGL_Data[EGL[id]]["ts"];
		-- elseif (EGL_Data_Cata[EGL[id]] ~= nil) then
			-- EGL_Skill_ID = EGL_Data[EGL[id]]["ts"];
		-- elseif (EGL_Data_JC[EGL[id]] ~= nil) then
			-- EGL_Skill_ID = EGL_Data[EGL[id]]["ts"];
		-- end
		-- if EGL_Skill_ID ~= nil then
			-- DoTradeSkill(EGL_Skill_ID,1);
			-- out("SkillID: "..EGL_Skill_ID);
		-- end
	-- end

end

-- JC CHECKBUTTON ------------------------------------------------

function EGL_CheckButton_OnClick(this)
	if (EGL_Options["jc"]) then
		EGL_Options["jc"] = false;
		EGL_Options["jc_cata"] = false;
		--out("EGL: Hide jewelcrafter gems ...");
		EGL_ScrollBar_Update();
	else
		EGL_Options["jc"] = true;
		if (EGL_Options["cataclysm"]) then
			EGL_Options["jc_cata"] = true;
		end
		--out("EGL: Show jewelcrafter gems ...");
		EGL_ScrollBar_Update();
	end
end

-- SLOT DROPDOWN ------------------------------------------------

function EGL_SlotDropDown_Init()
	local entries = table.getn(EGL_Slot);
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Slot[i];
		info.value = value;
		info.text = text;
		info.func = EGL_SlotDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_SlotDropDown_OnShow()
    UIDropDownMenu_Initialize(EGL_SlotDropDown, EGL_SlotDropDown_Init);
    UIDropDownMenu_SetSelectedID(EGL_SlotDropDown, EGL_Options["SelectedSlot"]+1);
    UIDropDownMenu_SetWidth(EGL_SlotDropDown, 70);
    EGL_ScrollBar_Update();
end

function EGL_SlotDropDown_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_SlotDropDown, i);
    EGL_Options["SelectedSlot"] = i-1;
    EGL_ScrollBar_Update();
end

-- STAT DROPDOWN ------------------------------------------------

function EGL_StatDropDown_Init()
	local entries = table.getn(EGL_Stat);
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Stat[i];
		info.text = text;
		info.value = value;
		info.func = EGL_StatDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_StatDropDown_OnShow()
    UIDropDownMenu_Initialize(EGL_StatDropDown, EGL_StatDropDown_Init);
    UIDropDownMenu_SetSelectedID(EGL_StatDropDown, EGL_Options["SelectedStat"]+1);
    UIDropDownMenu_SetWidth(EGL_StatDropDown, 150);
    EGL_ScrollBar_Update();
end

function EGL_StatDropDown_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_StatDropDown, i);
    EGL_Options["SelectedStat"] = i-1;
    EGL_ScrollBar_Update();
end

-- QUALI OP ------------------------------------------------

function EGL_QualiOp_Init()
	local entries = table.getn(EGL_Op);
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Op[i];
		info.value = value;
		info.text = text;
		info.func = EGL_QualiOp_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_QualiOp_OnShow()
    UIDropDownMenu_Initialize(EGL_QualiOp, EGL_QualiOp_Init);
    UIDropDownMenu_SetSelectedID(EGL_QualiOp, EGL_Options["SelectedQualiOp"]+1);
    UIDropDownMenu_SetWidth(EGL_QualiOp, 40);
    EGL_ScrollBar_Update();
end

function EGL_QualiOp_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_QualiOp, i);
    EGL_Options["SelectedQualiOp"] = i-1;
    EGL_ScrollBar_Update();
end

-- QUALITY DROPDOWN ------------------------------------------------

function EGL_QualiDropDown_Init()
	local entries = table.getn(EGL_Quali);
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Quali[i];
		info.text = text;
		info.value = value;
		info.func = EGL_QualiDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_QualiDropDown_OnShow()
    UIDropDownMenu_Initialize(EGL_QualiDropDown, EGL_QualiDropDown_Init);
    UIDropDownMenu_SetSelectedID(EGL_QualiDropDown, EGL_Options["SelectedQuali"]+1);
    UIDropDownMenu_SetWidth(EGL_QualiDropDown, 80);
    EGL_ScrollBar_Update();
end

function EGL_QualiDropDown_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_QualiDropDown, i);
    EGL_Options["SelectedQuali"] = i-1;
    EGL_ScrollBar_Update();
end

-- LEVEL OP ------------------------------------------------

function EGL_LevelOp_Init()
	local entries = table.getn(EGL_Op);
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Op[i];
		info.value = value;
		info.text = text;
		info.func = EGL_LevelOp_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_LevelOp_OnShow()
    UIDropDownMenu_Initialize(EGL_LevelOp, EGL_LevelOp_Init);
    UIDropDownMenu_SetSelectedID(EGL_LevelOp, EGL_Options["SelectedLevelOp"]+1);
    UIDropDownMenu_SetWidth(EGL_LevelOp, 40);
    EGL_ScrollBar_Update();
end

function EGL_LevelOp_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_LevelOp, i);
    EGL_Options["SelectedLevelOp"] = i-1;
    EGL_ScrollBar_Update();
end

-- ITEMLEVEL DROPDOWN ------------------------------------------------

function EGL_LevelDropDown_Init()
	local entries = table.getn(EGL_Level);
	if (EGL_Options["cataclysm"] == false) then
		entries = entries - 1;
	end
	for i=0,entries do
		info = {};
		value = i;
		text = EGL_Level[i];
		info.text = text;
		info.value = value;
		info.func = EGL_LevelDropDown_OnClick;
		UIDropDownMenu_AddButton(info);
	end
end

function EGL_LevelDropDown_OnShow()
    UIDropDownMenu_Initialize(EGL_LevelDropDown, EGL_LevelDropDown_Init);
    UIDropDownMenu_SetSelectedID(EGL_LevelDropDown, EGL_Options["SelectedLevel"]+1);
    UIDropDownMenu_SetWidth(EGL_LevelDropDown, 80);
    EGL_ScrollBar_Update();
end

function EGL_LevelDropDown_OnClick(this)
    i = this:GetID();
    UIDropDownMenu_SetSelectedID(EGL_LevelDropDown, i);
    EGL_Options["SelectedLevel"] = i-1;
    EGL_ScrollBar_Update();
end

-- RESULT LIST ------------------------------------------------

function EGL_HasNotStat(egltable,value)
	if EGL_Options["SelectedStat"] ~= 0 and 
	 (egltable[value]["stat1"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat2"] ~= EGL_Options["SelectedStat"] and egltable[value]["stat3"] ~= EGL_Options["SelectedStat"]) then
		return true;
	else
		return false;
	end

end

function EGL_IsNotSlot(egltable,value)
	if EGL_Options["SelectedSlot"] ~= 0 and 
	  ((EGL_Options["SelectedSlot"] == 1 and egltable[value]["red"] == false) or
	  (EGL_Options["SelectedSlot"] == 2 and egltable[value]["blue"] == false) or
	  (EGL_Options["SelectedSlot"] == 3 and egltable[value]["yellow"] == false) or
	  (EGL_Options["SelectedSlot"] == 4 and egltable[value]["meta"] == false)) then
		return true;
	else
		return false;
	end
end

function EGL_IsNotQuali(value)
	if EGL_Options["SelectedQuali"] ~= 0 then
		itemName, itemLink, itemRarity = GetItemInfo(value);
		if itemRarity == nil then
			return false;
		end
		if EGL_Options["SelectedQualiOp"] == 0 then
			if (itemRarity ~= (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 1 then
			if (itemRarity < (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		elseif EGL_Options["SelectedQualiOp"] == 2 then
			if (itemRarity > (EGL_Options["SelectedQuali"] + 1)) then
				return true;
			else
				return false;
			end
		end	
	end
	return false;
end

function EGL_ShowLevel(selected,required)
	if EGL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EGL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EGL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentgem;    
    EGL = {};    
    currentgem = 1;
	egl_shown_gems = 0;
	
	
	--if ((EGL_Options["SelectedLevel"] == 0) or (EGL_Options["SelectedLevel"] == 3)) then -- ALL, WotLK
	if (EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2) then -- ALL, WotLK
		for value,_ in pairs(EGL_Data) do    
			additem = true;	  
			if EGL_IsNotQuali(value) then
				additem = false;
			end
			if EGL_HasNotStat(EGL_Data,value) then
				additem = false;
			end			
			if EGL_IsNotSlot(EGL_Data,value) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end
	end
	
	if ((EGL_Options["cataclysm"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cataclysm
		for value,_ in pairs(EGL_Data_Cata) do    
			additem = true;	    
			if EGL_IsNotQuali(value) then
				additem = false;
			end
			if EGL_HasNotStat(EGL_Data_Cata,value) then
				additem = false;
			end			
			if EGL_IsNotSlot(EGL_Data_Cata,value) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end
	
	if ((EGL_Options["jc"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],2))) then	-- jc + ALL, WotLK
		for value,_ in pairs(EGL_Data_JC) do    
			additem = true;	    	    
			if EGL_IsNotQuali(value) then
				additem = false;
			end
			if EGL_HasNotStat(EGL_Data_JC,value) then
				additem = false;
			end			
			if EGL_IsNotSlot(EGL_Data_JC,value) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end

	if ((EGL_Options["jc_cata"]) and ((EGL_Options["SelectedLevel"] == 0) or EGL_ShowLevel(EGL_Options["SelectedLevel"],3))) then	-- if 4.0.3 + ALL, Cata
		for value,_ in pairs(EGL_Data_Cata_JC) do    
			additem = true;	    	    
			if EGL_IsNotQuali(value) then
				additem = false;
			end
			if EGL_HasNotStat(EGL_Data_Cata_JC,value) then
				additem = false;
			end			
			if EGL_IsNotSlot(EGL_Data_Cata_JC,value) then
				additem = false;
			end
			if additem == true then
				tinsert(EGL, value);
			end        
			currentgem = currentgem + 1;
		end      
	end	
    table.sort(EGL, EGL_SortResults);
    EGL_ShowGemList();    
end

function EGL_SortResults(a, b)
	local iteminfoA = GetItemInfo(a);
	local iteminfoB = GetItemInfo(b);
    local itemNameA, itemLinkA, itemQualityA, itemLevelA = GetItemInfo(a);
    local itemNameB, itemLinkB, itemQualityB, itemLevelB = GetItemInfo(b);
	
	if ((iteminfoA ~= nil) and (iteminfoB ~= nil)) then
		if itemLevelA ~= itemLevelB then	-- Sort by item level first
			return itemLevelA > itemLevelB;
		end
		if itemQualityA ~= itemQualityB then	-- Sort by item quality second
			return itemQualityA > itemQualityB;
		end
		if itemNameA and itemNameB then		-- if same item level, sort by name
			return itemNameA < itemNameB;
		end
	end
	return a < b;						-- sort by id, if no name
end


function EGL_ShowGemList()  
	local i, item, itemm, itemn, itemi;
	local sg = 15;
    MaxItems = table.getn(EGL);
    FauxScrollFrame_Update(EGL_ScrollBar,MaxItems,sg,16);
    for i=1,sg do
		item = _G["EGLItem_"..i];
		itemn = _G["EGLItem_"..i.."_Name"];
		itemi = _G["EGLItem_"..i.."_Icon"];
        lineplusoffset = i + FauxScrollFrame_GetOffset(EGL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(EGL[lineplusoffset]);
			if itemName then
				if itemn ~= nil then
					itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if item ~= nil then
					item:Show();
					--ScanTooltip:SetHyperlink("item:"..EGL[lineplusoffset]..":0:0:0");
					--EnumerateTooltipLines_helper(ScanTooltip:GetRegions());
				end
			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EGL_GEMCOUNT"];
	if itemm ~= nil then
		itemm:SetText(MaxItems.." gems:");
	end
end

-- INITIAL GEM SCAN ------------------------------------------------

function EGL_GetGemInfoDelayed(value)
	iteminfo = GetItemInfo(value);
	if iteminfo then
		EGL_ScrollBar_Update();
	else
		EGL_GetGemInfo(value);
	end
end


function EGL_GetGemInfo(value)
	iteminfo = GetItemInfo(value);
	if iteminfo then
		EGL_Scanned_Gems = EGL_Scanned_Gems + 1;
	else
		EGL_wait(5,EGL_GetGemInfoDelayed,value);
		failed = failed + 1;
	end
end

function EGL_GemScan()
	if egl_scan_state == 0 then
		EGL_Scanned_Gems = 0;
		out("EGL: Scanning gems ...")
		for value,_ in pairs(EGL_Data) do
			EGL_GetGemInfo(value);
		end
		for value,_ in pairs(EGL_Data_JC) do
			EGL_GetGemInfo(value);
		end
		if EGL_Options["cataclysm"] then
			for value,_ in pairs(EGL_Data_Cata) do
				EGL_GetGemInfo(value);
			end
			for value,_ in pairs(EGL_Data_Cata_JC) do
				EGL_GetGemInfo(value);
			end				
		end
	end
	EGL_FinalizeScan();
end

function EGL_AbortScan()
	egl_scan_state=0;
end

function EGL_ForceScan()
	egl_scan_state = 0;
	EGL_GemScan();
end

function EGL_FinalizeScan()
	egl_scan_state=1;
	out("EGL: Scanned "..EGL_Scanned_Gems.." gems.");
	if (failed > 0) then
		out("EGL: Missing "..failed.." gems will be added in a few seconds ...");
		failed = 0;
	end
	EGL_ScrollBar_Update();
end


